"""Elasticsearch library wrapper"""
from datetime import datetime
from typing import Any, Dict, Optional, Union
from src.constants.config import BasicConfig
from src.helpers.decorators import check_debug_mode, connect_elk, enable_retries
from src.helpers.formats import (
    check_log_level,
    format_elk_query_payload,
    format_log_data,
)
from src.loggers.interface import LoggerInterface


class Elk(LoggerInterface):
    """
    Elasticsearch library wrapper that inherits the LoggerInterface self variables and methods.
    """

    @check_debug_mode
    @connect_elk
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

    @check_debug_mode
    @enable_retries
    def log(
        self,
        level: str,
        logdata: Union[str, Dict[str, Any]],
        custom_func_name: Optional[str] = "",
        use_custom_logdata: Optional[bool] = False,
        date: Optional[str] = None,
    ):
        """Override inherited method from LoggerInterface"""
        check_log_level(level)

        self.es.index(
            index=self.index,
            body=format_log_data(
                self, level.upper(), logdata, custom_func_name, use_custom_logdata, date
            ),
        )

    async def async_log(self) -> None:
        """Not used"""
        raise NotImplementedError("Please use 'log' method instead.")

    @check_debug_mode
    @enable_retries
    def query(
        self,
        custompayload: Optional[Dict[str, Any]] = None,
        size: int = BasicConfig.DEFAULT_SIZE.value,
        cache: bool = True,
    ) -> Any:
        """Override inherited method from LoggerInterface"""
        return self.es.search(
            index=self.index,
            size=size,
            body=format_elk_query_payload(
                self.appname, datetime.now().isoformat(), custompayload
            ),
            request_cache=cache,
        )

    async def async_query(self) -> None:
        """Not used"""
        raise NotImplementedError("Please use 'log' method instead.")
